@@ -1,7 +1,7 @@
 /* Functionality for a bar across the bottom of the screen listing the
- * windows currently managed. 
+ * windows currently managed.
  *
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -68,21 +68,21 @@ hide_bar (rp_screen *s)
 
 /* Show window listing in bar. */
 int
-show_bar (rp_screen *s)
+show_bar (rp_screen *s, char *fmt)
 {
   if (!s->bar_is_raised)
     {
       s->bar_is_raised = BAR_IS_WINDOW_LIST;
       XMapRaised (dpy, s->bar_window);
-      update_window_names (s);
-  
+      update_window_names (s, fmt);
+
       reset_alarm();
       return 1;
     }
 
   /* If the bar is raised we still need to display the window
      names. */
-  update_window_names (s);
+  update_window_names (s, fmt);
   return 0;
 }
 
@@ -128,14 +128,14 @@ bar_y (rp_screen *s, int height)
     case EastGravity:
     case CenterGravity:
     case WestGravity:
-      y = s->top + (s->height - height 
-	   - defaults.bar_border_width * 2) / 2;
+      y = s->top + (s->height - height
+           - defaults.bar_border_width * 2) / 2;
       break;
     case SouthEastGravity:
     case SouthGravity:
     case SouthWestGravity:
-      y = s->top + (s->height - height 
-	   - defaults.bar_border_width * 2);
+      y = s->top + (s->height - height
+           - defaults.bar_border_width * 2);
       break;
     }
 
@@ -148,19 +148,11 @@ update_bar (rp_screen *s)
   if (s->bar_is_raised == BAR_IS_HIDDEN)
     return;
 
-  if (s->bar_is_raised == BAR_IS_MESSAGE)
-    {
-      show_last_message();
-    }
-  else
-    {
-      /* bar is showing a window list. */
-      update_window_names (s);
-    }
+  show_last_message();
 }
 
 void
-update_window_names (rp_screen *s)
+update_window_names (rp_screen *s, char *fmt)
 {
   struct sbuf *bar_buffer;
   int mark_start = 0;
@@ -172,12 +164,12 @@ update_window_names (rp_screen *s)
 
   if(defaults.window_list_style == STYLE_ROW)
     {
-      get_window_list (defaults.window_fmt, NULL, bar_buffer, &mark_start, &mark_end);
+      get_window_list (fmt, NULL, bar_buffer, &mark_start, &mark_end);
       marked_message (sbuf_get (bar_buffer), mark_start, mark_end);
     }
   else
     {
-      get_window_list (defaults.window_fmt, "\n", bar_buffer, &mark_start, &mark_end);
+      get_window_list (fmt, "\n", bar_buffer, &mark_start, &mark_end);
       marked_message (sbuf_get (bar_buffer), mark_start, mark_end);
     }
 
@@ -215,7 +207,7 @@ count_lines (char* msg, int len)
   if (len < 1)
     return 1;
 
-  for(i=0; i<len; i++) 
+  for(i=0; i<len; i++)
     {
       if (msg[i] == '\n') ret++;
     }
@@ -227,27 +219,27 @@ count_lines (char* msg, int len)
 static int
 max_line_length (char* msg)
 {
-  int i;
-  int start;
+  size_t i;
+  size_t start;
   int ret = 0;
-  
+
   /* Count each line and keep the length of the longest one. */
-  for(start=0, i=0; i <= strlen(msg); i++) 
+  for(start=0, i=0; i <= strlen(msg); i++)
     {
-      if(msg[i] == '\n' || msg[i] == '\0') 
-	{
-	  int current_width;
+      if(msg[i] == '\n' || msg[i] == '\0')
+        {
+          int current_width;
 
-	  /* Check if this line is the longest so far. */
-	  current_width = XTextWidth (defaults.font, msg + start, i - start);
-	  if(current_width > ret)
-	    {
-	      ret = current_width;
-	    }
+          /* Check if this line is the longest so far. */
+          current_width = XTextWidth (defaults.font, msg + start, i - start);
+          if(current_width > ret)
+            {
+              ret = current_width;
+            }
 
-	  /* Update the start of the new line. */
-	  start = i + 1;
-	}
+          /* Update the start of the new line. */
+          start = i + 1;
+        }
     }
 
   return ret;
@@ -267,8 +259,8 @@ pos_in_line (char* msg, int pos)
   ret = 0;
   for(i=pos-1; i>=0; ret++, i--)
     {
-      if(msg[i]=='\n') 
-	break;
+      if(msg[i]=='\n')
+        break;
     }
 
   return ret;
@@ -280,18 +272,18 @@ line_beginning (char* msg, int pos)
   int ret = 0;
   int i;
 
-  if(pos <= 0) 
+  if(pos <= 0)
     return 0;
 
   /* Go backwards until we hit a new line or the beginning of the
      string. */
-  for(i=pos-1; i>=0; --i) 
+  for(i=pos-1; i>=0; --i)
     {
       if (msg[i]=='\n')
-	{
-	  ret = i + 1;
-	  break;
-	}
+        {
+          ret = i + 1;
+          break;
+        }
     }
 
   return ret;
@@ -300,7 +292,7 @@ line_beginning (char* msg, int pos)
 static void
 draw_string (rp_screen *s, char *msg)
 {
-  int i;
+  size_t i;
   int line_no;
   int start;
   int line_height = FONT_HEIGHT (defaults.font);
@@ -308,28 +300,28 @@ draw_string (rp_screen *s, char *msg)
   /* Walk through the string, print each line. */
   start = 0;
   line_no = 0;
-  for(i=0; i < strlen(msg); ++i) 
+  for(i=0; i < strlen(msg); ++i)
     {
       /* When we encounter a new line, print the text up to the new
-	 line, and move down one line. */
+         line, and move down one line. */
       if (msg[i] == '\n')
-	{
-	  XDrawString (dpy, s->bar_window, s->normal_gc,
-		       defaults.bar_x_padding,
-		       defaults.bar_y_padding + defaults.font->max_bounds.ascent
-		       +  line_no * line_height,
-		       msg + start, i - start);
-	  line_no++;
-	  start = i + 1;
-	}
+        {
+          XDrawString (dpy, s->bar_window, s->normal_gc,
+                       defaults.bar_x_padding,
+                       defaults.bar_y_padding + defaults.font->max_bounds.ascent
+                       +  line_no * line_height,
+                       msg + start, i - start);
+          line_no++;
+          start = i + 1;
+        }
     }
-    
+
   /* Print the last line. */
   XDrawString (dpy, s->bar_window, s->normal_gc,
-	       defaults.bar_x_padding,
-	       defaults.bar_y_padding + defaults.font->max_bounds.ascent
-	       +  line_no * line_height,
-	       msg + start, strlen (msg) - start);
+               defaults.bar_x_padding,
+               defaults.bar_y_padding + defaults.font->max_bounds.ascent
+               +  line_no * line_height,
+               msg + start, strlen (msg) - start);
 
   XSync (dpy, False);
 }
@@ -367,9 +359,9 @@ correct_mark (int msg_len, int *mark_start, int *mark_end)
 static void
 prepare_bar (rp_screen *s, int width, int height)
 {
-  XMoveResizeWindow (dpy, s->bar_window, 
-		     bar_x (s, width), bar_y (s, height),
-		     width, height);
+  XMoveResizeWindow (dpy, s->bar_window,
+                     bar_x (s, width), bar_y (s, height),
+                     width, height);
 
   /* Map the bar if needed */
   if (!s->bar_is_raised)
@@ -384,8 +376,8 @@ prepare_bar (rp_screen *s, int width, int height)
 }
 
 static void
-get_mark_box (char *msg, int mark_start, int mark_end, 
-	      int *x, int *y, int *width, int *height)
+get_mark_box (char *msg, size_t mark_start, size_t mark_end,
+              int *x, int *y, int *width, int *height)
 {
   int start, end;
   int mark_end_is_new_line = 0;
@@ -414,20 +406,20 @@ get_mark_box (char *msg, int mark_start, int mark_end,
   end_line_beginning = line_beginning(msg, mark_end);
 
   PRINT_DEBUG (("start_line = %d, end_line = %d\n", start_line, end_line));
-  PRINT_DEBUG (("start_line_beginning = %d, end_line_beginning = %d\n", 
-		start_line_beginning, end_line_beginning));
+  PRINT_DEBUG (("start_line_beginning = %d, end_line_beginning = %d\n",
+                start_line_beginning, end_line_beginning));
 
   if (mark_start == 0 || start_pos_in_line == 0)
-    start = defaults.bar_x_padding;
+    start = 0;
   else
-    start = XTextWidth (defaults.font, 
-			&msg[start_line_beginning], 
-			start_pos_in_line) + defaults.bar_x_padding;
+    start = XTextWidth (defaults.font,
+                        &msg[start_line_beginning],
+                        start_pos_in_line) + defaults.bar_x_padding;
+
+  end = XTextWidth (defaults.font,
+                    &msg[end_line_beginning],
+                    end_pos_in_line) + defaults.bar_x_padding * 2;
 
-  end = XTextWidth (defaults.font, 
-		    &msg[end_line_beginning], 
-		    end_pos_in_line) + defaults.bar_x_padding * 2;
-      
   if (mark_end != strlen (msg))
     end -= defaults.bar_x_padding;
 
@@ -435,7 +427,7 @@ get_mark_box (char *msg, int mark_start, int mark_end,
      mark_end is at the end of a line. */
   if (mark_end_is_new_line)
     {
-      *width = max_line_length(msg);
+      *width = max_line_length(msg) + defaults.bar_x_padding * 2;
     }
   else
     {
@@ -454,20 +446,20 @@ draw_inverse_box (rp_screen *s, int x, int y, int width, int height)
   GC lgc;
   unsigned long mask;
 
-  lgv.foreground = current_screen()->fg_color;
+  lgv.foreground = s->fg_color;
   lgv.function = GXxor;
   mask = GCForeground | GCFunction;
   lgc = XCreateGC(dpy, s->root, mask, &lgv);
 
-  XFillRectangle (dpy, s->bar_window, lgc, 
-		  x, y, width, height);
+  XFillRectangle (dpy, s->bar_window, lgc,
+                  x, y, width, height);
   XFreeGC (dpy, lgc);
 
   lgv.foreground = s->bg_color;
   lgc = XCreateGC(dpy, s->root, mask, &lgv);
 
-  XFillRectangle (dpy, s->bar_window, lgc, 
-		  x, y, width, height);
+  XFillRectangle (dpy, s->bar_window, lgc,
+                  x, y, width, height);
   XFreeGC (dpy, lgc);
 }
 
@@ -480,8 +472,8 @@ draw_mark (rp_screen *s, char *msg, int mark_start, int mark_end)
   if (mark_end == 0 || mark_start == mark_end)
     return;
 
-  get_mark_box (msg, mark_start, mark_end, 
-		&x, &y, &width, &height);
+  get_mark_box (msg, mark_start, mark_end,
+                &x, &y, &width, &height);
   draw_inverse_box (s, x, y, width, height);
 }
 
