@@ -1,5 +1,5 @@
 /* functions for managing the program bar
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -22,9 +22,9 @@
 #ifndef _RATPOISON_BAR_H
 #define _RATPOISON_BAR_H 1
 
-void update_window_names (rp_screen *s);
+void update_window_names (rp_screen *s, char *fmt);
 void update_bar (rp_screen *s);
-int show_bar (rp_screen *s);
+int show_bar (rp_screen *s, char *fmt);
 int hide_bar (rp_screen *s);
 int bar_y (rp_screen *s, int height);
 int bar_x (rp_screen *s, int width);
