@@ -1,5 +1,5 @@
-/* Config file for ratpoison. Edit these values and recompile. 
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+/* Config file for ratpoison. Edit these values and recompile.
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -45,7 +45,7 @@
 #define RESIZE_VGROW_MODIFIER RP_CONTROL_MASK
 
 /* Key used to shrink frame vertically when in resize mode.  */
-#define RESIZE_VSHRINK_KEY	XK_p
+#define RESIZE_VSHRINK_KEY      XK_p
 #define RESIZE_VSHRINK_MODIFIER RP_CONTROL_MASK
 
 /* Key used to enlarge frame horizontally when in resize mode.  */
@@ -53,15 +53,15 @@
 #define RESIZE_HGROW_MODIFIER RP_CONTROL_MASK
 
 /* Key used to shrink frame horizontally when in resize mode.  */
-#define RESIZE_HSHRINK_KEY	XK_b
+#define RESIZE_HSHRINK_KEY      XK_b
 #define RESIZE_HSHRINK_MODIFIER RP_CONTROL_MASK
 
 /* Key used to shrink frame to fit it's current window.  */
-#define RESIZE_SHRINK_TO_WINDOW_KEY		XK_s
-#define RESIZE_SHRINK_TO_WINDOW_MODIFIER	0
+#define RESIZE_SHRINK_TO_WINDOW_KEY             XK_s
+#define RESIZE_SHRINK_TO_WINDOW_MODIFIER        0
 
 /* Key used to exit resize mode.  */
-#define RESIZE_END_KEY	    XK_Return
+#define RESIZE_END_KEY      XK_Return
 #define RESIZE_END_MODIFIER 0
 
 /* Number of history items to store. */
@@ -71,6 +71,10 @@
    (don't hide the windows underneath, and center them) */
 #define MAXSIZE_WINDOWS_ARE_TRANSIENTS
 
+/* Treat windows with aspect hints as if they were a transient window
+   (don't hide the windows underneath, and center them) */
+#define ASPECT_WINDOWS_ARE_TRANSIENTS
+
 /* An alias command could recursively call inself infinitely. This
    stops that behavior. */
 #define MAX_ALIAS_RECURSIVE_DEPTH 16
@@ -115,4 +119,7 @@
 /* The name of the top level keymap */
 #define TOP_KEYMAP "top"
 
+/* The default font */
+#define DEFAULT_FONT "9x15bold"
+
 #endif /* !_ _RATPOISON_CONF_H */
