@@ -1,5 +1,5 @@
-/* our datatypes and global variables 
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+/* our datatypes and global variables
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -50,7 +50,11 @@ struct rp_frame
 
   /* For determining the last frame. */
   int last_access;
-  
+
+  /* Boolean that is set when a frame is
+     `dedicated' (a.k.a. glued) to one window. */
+  unsigned int dedicated;
+
   struct list_head node;
 };
 
@@ -99,6 +103,10 @@ struct rp_window
      mapped into. */
   int frame_number;
 
+  /* Sometimes a window is intended for a certain frame. When a window
+     is mapped and this is >0 then use the frame (if it exists). */
+  int intended_frame_number;
+
   struct list_head node;
 };
 
@@ -139,15 +147,15 @@ struct rp_screen
   GC normal_gc;
   Window root, bar_window, key_window, input_window, frame_window, help_window;
   int bar_is_raised;
-  int screen_num;		/* Our screen number as dictated my X */
-  int xine_screen_num;		/* Our screen number for the Xinerama extension */
+  int screen_num;               /* Our screen number as dictated my X */
+  int xine_screen_num;          /* Our screen number for the Xinerama extension */
   Colormap def_cmap;
   Cursor rat;
   unsigned long fg_color, bg_color; /* The pixel color. */
-  
+
   /* Here to abstract over the Xinerama vs X screens difference */
   int left, top, width, height;
-  
+
   char *display_string;
 
   /* A list of frames that may or may not contain windows. There should
@@ -166,7 +174,7 @@ struct rp_action
 {
   KeySym key;
   unsigned int state;
-  void *data;			/* misc data to be passed to the function */
+  char *data;                   /* misc data to be passed to the function */
 /*   void (*func)(void *); */
 };
 
@@ -203,7 +211,7 @@ struct rp_defaults
   int bar_location;
   int bar_timeout;
   int bar_border_width;
-  
+
   int frame_indicator_timeout;
   int frame_resize_unit;
 
@@ -221,6 +229,7 @@ struct rp_defaults
   int wait_for_key_cursor;
 
   char *window_fmt;
+  char *info_fmt;
 
   /* Which name to use: wm_name, res_name, res_class. */
   int win_name;
@@ -237,6 +246,9 @@ struct rp_defaults
   int history_size;
 
   char *frame_selectors;
+
+  /* How many frame sets to remember when undoing. */
+  int maxundos;
 };
 
 /* Information about a child process. */
@@ -254,6 +266,16 @@ struct rp_child_info
   /* When this is != 0 then the process finished. */
   int terminated;
 
+  /* what was current when it was launched? */
+  rp_group *group;
+  rp_frame *frame;
+  rp_screen *screen;
+
+  /* Non-zero when the pid has mapped a window. This is to prevent
+     every window the program opens from getting mapped in the frame
+     it was launched from. Only the first window should do this. */
+  int window_mapped;
+
   /* This structure can exist in a list. */
   struct list_head node;
 };
@@ -263,10 +285,10 @@ struct rp_child_info
    comes to compare modifier masks. */
 #define RP_SHIFT_MASK   1
 #define RP_CONTROL_MASK 2
-#define RP_META_MASK 	4
-#define RP_ALT_MASK 	8
-#define RP_SUPER_MASK 	16
-#define RP_HYPER_MASK 	32
+#define RP_META_MASK    4
+#define RP_ALT_MASK     8
+#define RP_SUPER_MASK   16
+#define RP_HYPER_MASK   32
 
 struct modifier_info
 {
@@ -277,9 +299,9 @@ struct modifier_info
   unsigned int hyper_mod_mask;
 
   /* Keep track of these because they mess up the grab and should be
-     ignored. */  
+     ignored. */
   unsigned int num_lock_mask;
-  unsigned int scroll_lock_mask; 
+  unsigned int scroll_lock_mask;
 };
 
 typedef struct list_head *(*completion_fn)(char *string);
@@ -325,4 +347,19 @@ struct rp_hook_db_entry
   struct list_head *hook;
 };
 
+typedef struct rp_frame_undo rp_frame_undo;
+struct rp_frame_undo
+{
+  char *frames;
+  rp_screen *screen;
+  struct list_head node;
+};
+
+typedef struct rp_xselection rp_xselection;
+struct rp_xselection
+{
+  unsigned char *text;
+  int len;
+};
+
 #endif /* _RATPOISON_DATA_H */
