@@ -1,5 +1,5 @@
 /* functions that manipulate the frame structure.
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -7,12 +7,12 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * ratpoison is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
@@ -21,6 +21,8 @@
 
 #include "ratpoison.h"
 
+#include <string.h>
+
 int
 frame_left (rp_frame *frame)
 {
@@ -107,14 +109,27 @@ frame_move_down (rp_frame *frame, int amount)
   frame->y += amount;
 }
 
+static void
+init_frame (rp_frame *f)
+{
+  f->number = 0;
+  f->x = 0;
+  f->y = 0;
+  f->width = 0;
+  f->height = 0;
+  f->win_number = 0;
+  f->last_access = 0;
+  f->dedicated = 0;
+}
+
 rp_frame *
 frame_new (rp_screen *s)
 {
   rp_frame *f;
 
   f = xmalloc (sizeof (rp_frame));
+  init_frame(f);
   f->number = numset_request (s->frames_numset);
-  f->last_access = 0;
 
   return f;
 }
@@ -146,7 +161,7 @@ frame_copy (rp_frame *frame)
 }
 
 char *
-frame_dump (rp_frame *frame)
+frame_dump (rp_frame *frame, rp_screen *screen)
 {
   rp_window *win;
   char *tmp;
@@ -156,14 +171,17 @@ frame_dump (rp_frame *frame)
   win = find_window_number (frame->win_number);
 
   s = sbuf_new (0);
-  sbuf_printf (s, "%d %d %d %d %d %ld %d", 
-	       frame->number,
-	       frame->x,
-	       frame->y,
-	       frame->width,
-	       frame->height,
-	       win ? win->w:0,
-	       frame->last_access);
+  sbuf_printf (s, "(frame :number %d :x %d :y %d :width %d :height %d :screenw %d :screenh %d :window %ld :last-access %d :dedicated %d)", 
+               frame->number,
+               frame->x,
+               frame->y,
+               frame->width,
+               frame->height,
+	       screen->width,
+	       screen->height,
+               win ? win->w:0,
+               frame->last_access,
+               frame->dedicated);
 
   /* Extract the string and return it, and don't forget to free s. */
   tmp = sbuf_get (s);
@@ -171,26 +189,90 @@ frame_dump (rp_frame *frame)
   return tmp;
 }
 
+/* Used only by frame_read */
+#define read_slot(x) do { tmp = strtok (NULL, " "); x = strtol(tmp,NULL,10); } while(0)
+
 rp_frame *
-frame_read (char *str)
+frame_read (char *str, rp_screen *screen)
 {
-  Window w;
+  Window w = 0L;
   rp_window *win;
   rp_frame *f;
+  char *tmp, *dup;
+  int screen_width = -1;
+  int screen_height = -1;
 
+  /* Create a blank frame. */
   f = xmalloc (sizeof (rp_frame));
-  if (sscanf (str, "%d %d %d %d %d %ld %d", 
-	      &f->number,
-	      &f->x,
-	      &f->y,
-	      &f->width,
-	      &f->height,
-	      &w,
-	      &f->last_access) < 7)
+  init_frame(f);
+
+  PRINT_DEBUG(("parsing '%s'\n", str));
+
+  dup = xstrdup(str);
+  tmp = strtok (dup, " ");
+
+  /* Verify it starts with '(frame ' */
+  if (strcmp(tmp, "(frame"))
     {
+      PRINT_DEBUG(("Doesn't start with '(frame '\n"));
+      free (dup);
       free (f);
       return NULL;
     }
+  /* NOTE: there is no check to make sure each field was filled in. */
+  tmp = strtok(NULL, " ");
+  while (tmp)
+    {
+      if (!strcmp(tmp, ":number"))
+        read_slot(f->number);
+      else if (!strcmp(tmp, ":x"))
+        read_slot(f->x);
+      else if (!strcmp(tmp, ":y"))
+        read_slot(f->y);
+      else if (!strcmp(tmp, ":width"))
+        read_slot(f->width);
+      else if (!strcmp(tmp, ":height"))
+        read_slot(f->height);
+      else if (!strcmp(tmp, ":screenw"))
+	read_slot(screen_width);
+      else if (!strcmp(tmp, ":screenh"))
+	read_slot(screen_height);
+      else if (!strcmp(tmp, ":window"))
+        read_slot(w);
+      else if (!strcmp(tmp, ":last-access"))
+        read_slot(f->last_access);
+      else if (!strcmp(tmp, ":dedicated")) {
+  	/* f->dedicated is unsigned, so read into local variable. */
+        long dedicated;
+
+	read_slot(dedicated);
+        if (dedicated <= 0)
+          f->dedicated = 0;
+        else if (f->dedicated >= 1)
+          f->dedicated = 1;
+      }
+      else if (!strcmp(tmp, ")"))
+        break;
+      else
+        PRINT_ERROR(("Unknown slot %s\n", tmp));
+      /* Read the next token. */
+      tmp = strtok(NULL, " ");
+    }
+  if (tmp)
+    PRINT_ERROR(("Frame has trailing garbage\n"));
+  free (dup);
+
+  /* adjust x, y, width and height to a possible screen size change */
+  if (screen_width > 0)
+    {
+      f->x = (f->x*screen->width)/screen_width;
+      f->width = (f->width*screen->width)/screen_width;
+    }
+  if (screen_height > 0)
+    {
+      f->y = (f->y*screen->height)/screen_height;
+      f->height = (f->height*screen->height)/screen_height;
+    }
 
   /* Perform some integrity checks on what we got and fix any
      problems. */
@@ -210,13 +292,11 @@ frame_read (char *str)
   /* Find the window with the X11 window ID. */
   win = find_window_in_list (w, &rp_mapped_window);
   if (win)
-    {
-      f->win_number = win->number;
-    }
+    f->win_number = win->number;
   else
-    {
-      f->win_number = EMPTY;
-    }
+    f->win_number = EMPTY;
 
   return f;
 }
+
+#undef read_slot
