@@ -1,5 +1,5 @@
 /* Ratpoison.
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -7,12 +7,12 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * ratpoison is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
@@ -45,15 +45,15 @@
 #endif
 
 /* Command line options */
-static struct option ratpoison_longopts[] = 
-  { {"help", 		no_argument, 		0, 	'h'},
-    {"interactive", 	no_argument, 		0, 	'i'},
-    {"version", 	no_argument, 		0, 	'v'},
-    {"command", 	required_argument, 	0, 	'c'},
-    {"display", 	required_argument, 	0, 	'd'},
-    {"screen",  	required_argument,      0,      's'},
-    {"file",  		required_argument,      0,      'f'},
-    {0, 		0, 			0, 	0} };
+static struct option ratpoison_longopts[] =
+  { {"help",            no_argument,            0,      'h'},
+    {"interactive",     no_argument,            0,      'i'},
+    {"version", no_argument,            0,      'v'},
+    {"command", required_argument,      0,      'c'},
+    {"display", required_argument,      0,      'd'},
+    {"screen",  required_argument,      0,      's'},
+    {"file",            required_argument,      0,      'f'},
+    {0,         0,                      0,      0} };
 
 static char ratpoison_opts[] = "hvic:d:s:f:";
 
@@ -112,9 +112,9 @@ xvsprintf (char *fmt, va_list ap)
       __va_copy (ap_copy, ap);
 #else
       /* If there is no copy macro then this MAY work. On some systems
-	 this could fail because va_list is a pointer so assigning one
-	 to the other as below wouldn't make a copy of the data, but
-	 just the pointer to the data. */
+         this could fail because va_list is a pointer so assigning one
+         to the other as below wouldn't make a copy of the data, but
+         just the pointer to the data. */
       ap_copy = ap;
 #endif
       nchars = vsnprintf (buffer, size, fmt, ap_copy);
@@ -123,11 +123,11 @@ xvsprintf (char *fmt, va_list ap)
 #endif
 
       if (nchars > -1 && nchars < size)
-	return buffer;
+        return buffer;
       else if (nchars > -1)
-	size = nchars + 1;
+        size = nchars + 1;
       else
-	size *= 2;
+        size *= 2;
 
       /* Resize the buffer and try again. */
       buffer = (char *)xrealloc (buffer, size);
@@ -181,36 +181,44 @@ alrm_handler (int signum)
   alarm_signalled++;
 }
 
+/* Check for child processes that have quit but haven't been
+   acknowledged yet. Update their structure. */
 void
-chld_handler (int signum)
+check_child_procs ()
 {
   rp_child_info *cur;
-  int pid, status, serrno;
-
-  serrno = errno;
-
+  int pid, status;
   while (1)
     {
       pid = waitpid (WAIT_ANY, &status, WNOHANG);
       if (pid <= 0)
-	break;
+        break;
 
       PRINT_DEBUG(("Child status: %d\n", WEXITSTATUS (status)));
 
       /* Find the child and update its structure. */
       list_for_each_entry (cur, &rp_children, node)
-	{
-	  if (cur->pid == pid)
-	    {
-	      cur->terminated = 1;
-	      cur->status = WEXITSTATUS (status);
-	      break;
-	    }
-	}
+        {
+          if (cur->pid == pid)
+            {
+              cur->terminated = 1;
+              cur->status = WEXITSTATUS (status);
+              break;
+            }
+        }
 
       chld_signalled = 1;
     }
-  errno = serrno;  
+}
+
+void
+chld_handler (int signum)
+{
+  int serrno;
+
+  serrno = errno;
+  check_child_procs();
+  errno = serrno;
 }
 
 int
@@ -218,11 +226,11 @@ handler (Display *d, XErrorEvent *e)
 {
   char error_msg[100];
 
-  if (e->request_code == X_ChangeWindowAttributes && e->error_code == BadAccess) 
+  if (e->request_code == X_ChangeWindowAttributes && e->error_code == BadAccess)
     {
       fprintf(stderr, "ratpoison: There can be only ONE.\n");
       exit(EXIT_FAILURE);
-    }  
+    }
 
 #ifdef IGNORE_BADWINDOW
   return 0;
@@ -235,7 +243,7 @@ handler (Display *d, XErrorEvent *e)
 
   /* If there is already an error to report, replace it with this new
      one. */
-  if (rp_error_msg) 
+  if (rp_error_msg)
     free (rp_error_msg);
   rp_error_msg = xstrdup (error_msg);
 
@@ -249,26 +257,26 @@ set_sig_handler (int sig, void (*action)(int))
     handler by default which is a tip of the hat to some god-aweful
     ancient code.  So use the POSIX sigaction call instead. */
   struct sigaction act;
-       
+
   /* check setting for sig */
-  if (sigaction (sig, NULL, &act)) 
+  if (sigaction (sig, NULL, &act))
     {
       PRINT_ERROR (("Error %d fetching SIGALRM handler\n", errno ));
-    } 
-  else 
+    }
+  else
     {
       /* if the existing action is to ignore then leave it intact
-	 otherwise add our handler */
-      if (act.sa_handler != SIG_IGN) 
-	{
-	  act.sa_handler = action;
-	  sigemptyset(&act.sa_mask);
-	  act.sa_flags = 0;
-	  if (sigaction (sig, &act, NULL)) 
-	    {
-	      PRINT_ERROR (("Error %d setting SIGALRM handler\n", errno ));
-	    }
-	}
+         otherwise add our handler */
+      if (act.sa_handler != SIG_IGN)
+        {
+          act.sa_handler = action;
+          sigemptyset(&act.sa_mask);
+          act.sa_flags = 0;
+          if (sigaction (sig, &act, NULL))
+            {
+              PRINT_ERROR (("Error %d setting SIGALRM handler\n", errno ));
+            }
+        }
     }
 }
 
@@ -276,10 +284,10 @@ void
 print_version ()
 {
   printf ("%s %s\n", PACKAGE, VERSION);
-  printf ("Copyright (C) 2000, 2001 Shawn Betts\n\n");
+  printf ("Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts\n\n");
 
   exit (EXIT_SUCCESS);
-}  
+}
 
 void
 print_help ()
@@ -293,7 +301,7 @@ print_help ()
   printf ("-i, --interactive     Execute commands in interactive mode\n");
   printf ("-f, --file <file>     Specify an alternative configuration file\n\n");
 
-  printf ("Report bugs to ratpoison-devel@lists.sourceforge.net\n\n");
+  printf ("Report bugs to ratpoison-devel@nongnu.org\n\n");
 
   exit (EXIT_SUCCESS);
 }
@@ -313,7 +321,7 @@ read_rc_file (FILE *file)
   size_t n = 256;
   char *partial;
   char *line;
-  int linesize = n;
+  size_t linesize = n;
 
   partial = (char*)xmalloc(n);
   line = (char*)xmalloc(linesize);
@@ -322,39 +330,39 @@ read_rc_file (FILE *file)
   while (fgets (partial, n, file) != NULL)
     {
       if ((strlen (line) + strlen (partial)) >= linesize)
-	{
-	  linesize *= 2;
-	  line = (char*) xrealloc (line, linesize);
-	}
+        {
+          linesize *= 2;
+          line = (char*) xrealloc (line, linesize);
+        }
 
       strcat (line, partial);
 
       if (feof(file) || (*(line + strlen(line) - 1) == '\n'))
-	{
-	  /* FIXME: this is a hack, command() should properly parse
-	     the command and args (ie strip whitespace, etc) 
+        {
+          /* FIXME: this is a hack, command() should properly parse
+             the command and args (ie strip whitespace, etc)
 
-	     We should not care if there is a newline (or vertical
-	     tabs or linefeeds for that matter) at the end of the
-	     command (or anywhere between tokens). */
-	  if (*(line + strlen(line) - 1) == '\n')
-	    *(line + strlen(line) - 1) = '\0';
+             We should not care if there is a newline (or vertical
+             tabs or linefeeds for that matter) at the end of the
+             command (or anywhere between tokens). */
+          if (*(line + strlen(line) - 1) == '\n')
+            *(line + strlen(line) - 1) = '\0';
 
-	  PRINT_DEBUG (("rcfile line: %s\n", line));
+          PRINT_DEBUG (("rcfile line: %s\n", line));
 
-	  /* do it */
-	  if (*line != '#')
-	    {
-	      char *result;
-	      result = command (0, line);
+          /* do it */
+          if (*line != '#')
+            {
+              cmdret *result;
+              result = command (0, line);
 
-	      /* Gobble the result. */
-	      if (result)
-		free (result);
-	    }
+              /* Gobble the result. */
+              if (result)
+                cmdret_free (result);
+            }
 
-	  *line = '\0';
-	}
+          *line = '\0';
+        }
     }
 
 
@@ -371,39 +379,39 @@ read_startup_files (char *alt_rcfile)
   if (alt_rcfile)
     {
       if ((fileptr = fopen (alt_rcfile, "r")) == NULL)
-	{
-	  /* we probably don't need to report this, its not an error */
-	  PRINT_DEBUG (("ratpoison: could not open %s\n", alt_rcfile));
-	}
+        {
+          /* we probably don't need to report this, its not an error */
+          PRINT_DEBUG (("ratpoison: could not open %s\n", alt_rcfile));
+        }
     }
   else
     {
       /* first check $HOME/.ratpoisonrc and if that does not exist then try
-	 /etc/ratpoisonrc */
+         /etc/ratpoisonrc */
 
       homedir = getenv ("HOME");
       if (!homedir)
-	{
-	  PRINT_ERROR (("ratpoison: $HOME not set!?\n"));
-	}
+        {
+          PRINT_ERROR (("ratpoison: $HOME not set!?\n"));
+        }
       else
-	{
-	  char *filename = (char*)xmalloc (strlen (homedir) + strlen ("/.ratpoisonrc") + 1);
-	  sprintf (filename, "%s/.ratpoisonrc", homedir);
-      
-	  if ((fileptr = fopen (filename, "r")) == NULL)
-	    {
-	      /* we probably don't need to report this, its not an error */
-	      PRINT_DEBUG (("ratpoison: could not open %s\n", filename));
+        {
+          char *filename;
+          filename = xsprintf ("%s/.ratpoisonrc", homedir);
 
-	      if ((fileptr = fopen ("/etc/ratpoisonrc", "r")) == NULL)
-		{
-		  /* neither is this */
-		  PRINT_DEBUG (("ratpoison: could not open /etc/ratpoisonrc\n"));
-		}
-	    }
-	  free (filename);
-	}
+          if ((fileptr = fopen (filename, "r")) == NULL)
+            {
+              /* we probably don't need to report this, its not an error */
+              PRINT_DEBUG (("ratpoison: could not open %s\n", filename));
+
+              if ((fileptr = fopen ("/etc/ratpoisonrc", "r")) == NULL)
+                {
+                  /* neither is this */
+                  PRINT_DEBUG (("ratpoison: could not open /etc/ratpoisonrc\n"));
+                }
+            }
+          free (filename);
+        }
     }
 
   if (fileptr)
@@ -422,10 +430,10 @@ show_welcome_message ()
   rp_action *help_action;
   char *prefix, *help;
   rp_keymap *map;
-  
+
   prefix = keysym_to_string (prefix_key.sym, prefix_key.state);
 
-  map = find_keymap (ROOT_KEYMAP); 
+  map = find_keymap (ROOT_KEYMAP);
 
   /* Find the help key binding. */
   help_action = find_keybinding_by_action ("help " ROOT_KEYMAP, map);
@@ -438,11 +446,11 @@ show_welcome_message ()
   if (help)
     {
       /* A little kludge to use ? instead of `question' for the help
-	 key. */
+         key. */
       if (!strcmp (help, "question"))
-	marked_message_printf (0, 0, MESSAGE_WELCOME, prefix, "?");
+        marked_message_printf (0, 0, MESSAGE_WELCOME, prefix, "?");
       else
-	marked_message_printf (0, 0, MESSAGE_WELCOME, prefix, help);
+        marked_message_printf (0, 0, MESSAGE_WELCOME, prefix, help);
 
       free (help);
     }
@@ -450,7 +458,7 @@ show_welcome_message ()
     {
       marked_message_printf (0, 0, MESSAGE_WELCOME, prefix, ":help");
     }
-  
+
   free (prefix);
 }
 
@@ -463,10 +471,10 @@ init_defaults ()
 
   defaults.input_window_size   = 200;
   defaults.window_border_width = 1;
-  defaults.bar_x_padding       = 0;
+  defaults.bar_x_padding       = 4;
   defaults.bar_y_padding       = 0;
   defaults.bar_location        = NorthEastGravity;
-  defaults.bar_timeout 	       = 5;
+  defaults.bar_timeout         = 5;
   defaults.bar_border_width    = 1;
 
   defaults.frame_indicator_timeout = 1;
@@ -474,16 +482,16 @@ init_defaults ()
 
   defaults.padding_left   = 0;
   defaults.padding_right  = 0;
-  defaults.padding_top 	  = 0;
+  defaults.padding_top    = 0;
   defaults.padding_bottom = 0;
 
-  defaults.font = XLoadQueryFont (dpy, "9x15bold");
+  defaults.font = XLoadQueryFont (dpy, DEFAULT_FONT);
   if (defaults.font == NULL)
     {
-      fprintf (stderr, "ratpoison: Cannot load font %s.\n", "9x15bold");
+      fprintf (stderr, "ratpoison: Cannot load font %s.\n", DEFAULT_FONT);
       exit (EXIT_FAILURE);
     }
-  defaults.font_string = xstrdup ("9x15bold");
+  defaults.font_string = xstrdup (DEFAULT_FONT);
 
   defaults.fgcolor_string = xstrdup ("black");
   defaults.bgcolor_string = xstrdup ("white");
@@ -491,14 +499,16 @@ init_defaults ()
   defaults.wait_for_key_cursor = 1;
 
   defaults.window_fmt = xstrdup ("%n%s%t");
+  defaults.info_fmt = xstrdup ("(%H, %W) %n(%t)");
 
   defaults.win_name = WIN_NAME_TITLE;
   defaults.startup_message = 1;
-  defaults.warp = 1;
+  defaults.warp = 0;
   defaults.window_list_style = STYLE_COLUMN;
 
   defaults.history_size = 20;
   defaults.frame_selectors = xstrdup ("");
+  defaults.maxundos = 20;
 }
 
 int
@@ -525,44 +535,54 @@ main (int argc, char *argv[])
       if (c == -1) break;
 
       switch (c)
-	{
-	case 'h':
-	  print_help ();
-	  break;
-	case 'v':
-	  print_version ();
-	  break;
-	case 'c':
-	  if (!command)
-	    {
-	      command = xmalloc (sizeof(char *));
-	      cmd_count = 0;
-	    }
-	  else
-	    {
-	      command = xrealloc (command, sizeof (char *) * (cmd_count + 1));
-	    }
+        {
+        case 'h':
+          print_help ();
+          break;
+        case 'v':
+          print_version ();
+          break;
+        case 'c':
+          if (!command)
+            {
+              command = xmalloc (sizeof(char *));
+              cmd_count = 0;
+            }
+          else
+            {
+              command = xrealloc (command, sizeof (char *) * (cmd_count + 1));
+            }
 
-	  command[cmd_count] = xstrdup (optarg);
-	  cmd_count++;
-	  break;
-	case 'd':
-	  display = xstrdup (optarg);
-	  break;
-	case 's':
-	  screen_arg = 1;
-	  screen_num = strtol (optarg, NULL, 10);
-	  break;
-	case 'i':
-	  interactive = 1;
-	  break;
-	case 'f':
-	  alt_rcfile = xstrdup (optarg);
-	  break;
-	  
-	default:
-	  exit (EXIT_FAILURE);
-	}
+          command[cmd_count] = xstrdup (optarg);
+          cmd_count++;
+          break;
+        case 'd':
+          display = xstrdup (optarg);
+          break;
+        case 's':
+          screen_arg = 1;
+          screen_num = strtol (optarg, NULL, 10);
+          break;
+        case 'i':
+          interactive = 1;
+          break;
+        case 'f':
+          alt_rcfile = xstrdup (optarg);
+          break;
+
+        default:
+          exit (EXIT_FAILURE);
+        }
+    }
+
+  /* Report extra unparsed arguments. */
+  if (optind < argc)
+    {
+      fprintf (stderr, "Error: junk arguments: ");
+      while (optind < argc)
+        fprintf (stderr, "%s ", argv[optind++]);
+      fputc ('\n', stderr);
+      exit (EXIT_FAILURE);
     }
 
   if (!(dpy = XOpenDisplay (display)))
@@ -582,14 +602,14 @@ main (int argc, char *argv[])
       int i;
 
       for (i=0; i<cmd_count; i++)
-	{
-	  if (screen_arg)
-	    send_command (interactive, command[i], screen_num);
-	  else
-	    send_command (interactive, command[i], -1);
+        {
+          if (screen_arg)
+            send_command (interactive, (unsigned char *)command[i], screen_num);
+          else
+            send_command (interactive, (unsigned char *)command[i], -1);
 
-	  free (command[i]);
-	}
+          free (command[i]);
+        }
 
       free (command);
       XCloseDisplay (dpy);
@@ -605,15 +625,25 @@ main (int argc, char *argv[])
   wm_take_focus = XInternAtom(dpy, "WM_TAKE_FOCUS", False);
   wm_colormaps = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);
 
+  /* netwm atoms */
+  _net_wm_pid = XInternAtom(dpy, "_NET_WM_PID", False);
+  PRINT_DEBUG (("_NET_WM_PID = %ld\n", _net_wm_pid));
+  _net_supported = XInternAtom(dpy, "_NET_SUPPORTED", False);
+  PRINT_DEBUG (("_NET_SUPPORTED = %ld\n", _net_supported));
+
   /* Setup signal handlers. */
-  XSetErrorHandler(handler);  
+  XSetErrorHandler(handler);
   set_sig_handler (SIGALRM, alrm_handler);
   set_sig_handler (SIGTERM, sighandler);
   set_sig_handler (SIGINT, sighandler);
   set_sig_handler (SIGHUP, hup_handler);
   set_sig_handler (SIGCHLD, chld_handler);
 
+  /* Add RATPOISON to the environment */
+  putenv (xsprintf ("RATPOISON=%s", argv[0]));
+
   /* Setup ratpoison's internal structures */
+  init_globals ();
   init_defaults ();
   init_groups ();
   init_window_stuff ();
@@ -622,10 +652,11 @@ main (int argc, char *argv[])
 
   init_frame_lists ();
   update_modifier_map ();
+  init_user_commands();
   initialize_default_keybindings ();
-#ifdef HAVE_READLINE_HISTORY_H
+#ifdef HAVE_HISTORY
   history_load ();
-#endif /* HAVE_READLINE_HISTORY_H */
+#endif /* HAVE_HISTORY */
 
   /* Scan for windows */
   if (screen_arg)
@@ -637,9 +668,9 @@ main (int argc, char *argv[])
     {
       rp_current_screen = 0;
       for (i=0; i<num_screens; i++)
-	{
-	  scanwins (&screens[i]);
-	}
+        {
+          scanwins (&screens[i]);
+        }
     }
 
   read_startup_files (alt_rcfile);
@@ -665,11 +696,14 @@ free_screen (rp_screen *s)
   rp_frame *frame;
   struct list_head *iter, *tmp;
 
+  /* Relinquish our hold on the root window. */
+  XSelectInput(dpy, RootWindow (dpy, s->screen_num), 0);
+
   list_for_each_safe_entry (frame, iter, tmp, &s->frames, node)
     {
       frame_free (s, frame);
     }
- 
+
   XDestroyWindow (dpy, s->bar_window);
   XDestroyWindow (dpy, s->key_window);
   XDestroyWindow (dpy, s->input_window);
@@ -688,9 +722,9 @@ clean_up ()
 {
   int i;
 
-#ifdef HAVE_READLINE_HISTORY_H
+#ifdef HAVE_HISTORY
   history_save ();
-#endif /* HAVE_READLINE_HISTORY_H */
+#endif /* HAVE_HISTORY */
 
   free_keymaps ();
   free_aliases ();
@@ -698,15 +732,24 @@ clean_up ()
   free_groups ();
 
   free_window_stuff ();
-  
+
   for (i=0; i<num_screens; i++)
     {
       free_screen (&screens[i]);
     }
   free (screens);
 
+  /* Delete the undo histories */
+  while (list_size (&rp_frame_undos) > 0)
+    {
+      /* Delete the oldest node */
+      rp_frame_undo *cur;
+      list_last (cur, &rp_frame_undos, node);
+      del_frame_undo (cur);
+    }
+
   /* Free the global frame numset shared by all screens. */
-  free (rp_frame_numset);
+  numset_free (rp_frame_numset);
 
   free_xinerama();
 
@@ -716,4 +759,3 @@ clean_up ()
   XSetInputFocus (dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
   XCloseDisplay (dpy);
 }
-
