@@ -1,5 +1,5 @@
 /* Standard header for ratpoison.
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -7,12 +7,12 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * ratpoison is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
@@ -29,7 +29,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <string.h>
 #include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #include <fcntl.h>
 
 /* Some systems don't define the close-on-exec flag in fcntl.h */
@@ -41,18 +43,18 @@
 #define PRINT_LINE(type) printf (PACKAGE ":%s:%d: %s: ",__FILE__,  __LINE__, #type)
 
 /* Error and debug reporting macros. */
-#define PRINT_ERROR(fmt)			\
-do {						\
-  PRINT_LINE (error);				\
-  printf fmt;					\
-  fflush (stdout);				\
+#define PRINT_ERROR(fmt)                        \
+do {                                            \
+  PRINT_LINE (error);                           \
+  printf fmt;                                   \
+  fflush (stdout);                              \
 } while (0)
 
 #ifdef DEBUG
-#define PRINT_DEBUG(fmt)			\
-do {						\
-  PRINT_LINE (debug);				\
-  printf fmt;					\
+#define PRINT_DEBUG(fmt)                        \
+do {                                            \
+  PRINT_LINE (debug);                           \
+  printf fmt;                                   \
   fflush (stdout);                              \
 } while (0)
 #else
@@ -79,12 +81,13 @@ extern XGCValues gv;
 #include "screen.h"
 #include "group.h"
 #include "editor.h"
-#ifdef HAVE_READLINE_HISTORY_H
+#ifdef HAVE_HISTORY
 #include "history.h"
-#endif /* HAVE_READLINE_HISTORY_H */
+#endif /* HAVE_HISTORY */
 #include "completions.h"
 #include "hook.h"
 #include "xinerama.h"
+#include "format.h"
 
 void clean_up ();
 rp_screen *find_screen (Window w);
@@ -99,5 +102,9 @@ char *xstrdup (char *s);
 char *xsprintf (char *fmt, ...);
 char *xvsprintf (char *fmt, va_list ap);
 int str_comp (char *s1, char *s2, int len);
+/* Needed in cmd_tmpwm */
+void check_child_procs ();
+void chld_handler (int signum);
+void set_sig_handler (int sig, void (*action)(int));
 
 #endif /* ! _RATPOISON_H */
