@@ -1,5 +1,5 @@
-/* functions for handling the window list 
- * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
+/* functions for handling the window list
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004 Shawn Betts <sabetts@vcn.bc.ca>
  *
  * This file is part of ratpoison.
  *
@@ -19,6 +19,12 @@
  * Boston, MA 02111-1307 USA
  */
 
+/* Citing getsid(2) here: 
+   To get the prototype under glibc, define both _XOPEN_SOURCE and
+   _XOPEN_SOURCE_EXTENDED, or use "#define _XOPEN_SOURCE n" for some
+   integer n larger than or equal to 500. */
+#define _XOPEN_SOURCE 500
+#include <unistd.h>             /* for getsid */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,15 +36,15 @@ LIST_HEAD(rp_mapped_window);
 
 struct numset *rp_window_numset;
 
-/* Get the mouse position relative to the root of the specified window */
+/* Get the mouse position relative to the the specified window */
 static void
-get_mouse_root_position (rp_window *win, int *mouse_x, int *mouse_y)
+get_mouse_position (rp_window *win, int *mouse_x, int *mouse_y)
 {
   Window root_win, child_win;
-  int win_x, win_y;
+  int root_x, root_y;
   unsigned int mask;
-  
-  XQueryPointer (dpy, win->scr->root, &root_win, &child_win, &win_x, &win_y, mouse_x, mouse_y, &mask);
+
+  XQueryPointer (dpy, win->scr->root, &root_win, &child_win, mouse_x, mouse_y, &root_x, &root_y, &mask);
 }
 
 void
@@ -62,11 +68,11 @@ update_window_gravity (rp_window *win)
 /*   if (win->hints->win_gravity == ForgetGravity) */
 /*     { */
       if (win->transient)
-	win->gravity = defaults.trans_gravity;
-      else if (win->hints->flags & PMaxSize)
-	win->gravity = defaults.maxsize_gravity;
+        win->gravity = defaults.trans_gravity;
+      else if (win->hints->flags & PMaxSize || win->hints->flags & PAspect)
+        win->gravity = defaults.maxsize_gravity;
       else
-	win->gravity = defaults.win_gravity;
+        win->gravity = defaults.win_gravity;
 /*     } */
 /*   else */
 /*     { */
@@ -86,30 +92,90 @@ window_name (rp_window *win)
     {
     case WIN_NAME_RES_NAME:
       if (win->res_name)
-	return win->res_name;
+        return win->res_name;
       else return win->user_name;
-      
+
     case WIN_NAME_RES_CLASS:
       if (win->res_class)
-	return win->res_class;
+        return win->res_class;
       else return win->user_name;
 
       /* if we're not looking for the res name or res class, then
-	 we're looking for the window title. */
+         we're looking for the window title. */
     default:
       if (win->wm_name)
-	return win->wm_name;
+        return win->wm_name;
       else return win->user_name;
     }
 
   return NULL;
 }
 
+/* FIXME: we need to verify that the window is running on the same
+   host as something. otherwise there could be overlapping PIDs. */
+struct rp_child_info *
+get_child_info (Window w)
+{
+  rp_child_info *cur;
+  int status;
+  int pid;
+  Atom type_ret;
+  int format_ret;
+  unsigned long nitems;
+  unsigned long bytes_after;
+  unsigned char *req;
+  pid_t sid;
+
+  status = XGetWindowProperty (dpy, w, _net_wm_pid,
+                               0, 0, False, XA_CARDINAL,
+                               &type_ret, &format_ret, &nitems, &bytes_after, &req);
+
+  if (status != Success || req == NULL)
+    {
+      PRINT_DEBUG (("Couldn't get _NET_WM_PID Property\n"));
+      return NULL;
+    }
+
+  /* XGetWindowProperty always allocates one extra byte even if
+     the property is zero length. */
+  XFree (req);
+
+  status = XGetWindowProperty (dpy, w, _net_wm_pid,
+                               0, (bytes_after / 4) + (bytes_after % 4 ? 1 : 0),
+                               False, XA_CARDINAL, &type_ret, &format_ret, &nitems,
+                               &bytes_after, &req);
+
+  if (status != Success || req == NULL)
+    {
+      PRINT_DEBUG (("Couldn't get _NET_WM_PID Property\n"));
+      return NULL;
+    }
+
+  pid = *((int *)req);
+  XFree(req);
+
+  PRINT_DEBUG(("pid: %d\n", pid));
+
+  /* The pids will hopefully be in the same session. */
+  sid = getsid (pid);
+  list_for_each_entry (cur, &rp_children, node)
+    {
+      PRINT_DEBUG(("cur->pid=%d sid=%d\n", cur->pid, getsid (cur->pid)));
+      if (sid == getsid (cur->pid))
+        return cur;
+    }
+
+  return NULL;
+}
+
 /* Allocate a new window and add it to the list of managed windows */
 rp_window *
 add_to_window_list (rp_screen *s, Window w)
 {
+  struct rp_child_info *child_info;
   rp_window *new_window;
+  rp_group *group = NULL;
+  int frame_num = -1;
 
   new_window = xmalloc (sizeof (rp_window));
 
@@ -119,6 +185,7 @@ add_to_window_list (rp_screen *s, Window w)
   new_window->state = WithdrawnState;
   new_window->number = -1;
   new_window->frame_number = EMPTY;
+  new_window->intended_frame_number = -1;
   new_window->named = 0;
   new_window->hints = XAllocSizeHints ();
   new_window->colormap = DefaultColormap (dpy, s->screen_num);
@@ -127,7 +194,7 @@ add_to_window_list (rp_screen *s, Window w)
 
   update_window_gravity (new_window);
 
-  get_mouse_root_position (new_window, &new_window->mouse_x, &new_window->mouse_y);
+  get_mouse_position (new_window, &new_window->mouse_x, &new_window->mouse_y);
 
   XSelectInput (dpy, new_window->w, WIN_EVENTS);
 
@@ -140,8 +207,29 @@ add_to_window_list (rp_screen *s, Window w)
   /* Add the window to the end of the unmapped list. */
   list_add_tail (&new_window->node, &rp_unmapped_window);
 
-  /* Add the window to the current group. */
-  group_add_window (rp_current_group, new_window);
+  child_info = get_child_info (w);
+
+  if (child_info && !child_info->window_mapped) {
+    rp_frame *frame = screen_find_frame_by_frame (child_info->screen, child_info->frame);
+
+    PRINT_DEBUG(("frame=%p\n", frame));
+    group = groups_find_group_by_group (child_info->group);
+    if (frame)
+      frame_num = frame->number;
+    /* Only map the first window in the launch frame. */
+    child_info->window_mapped = 1;
+  }
+
+  /* Add the window to the group it's pid was launched in or the
+     current one. */
+  if (group)
+    group_add_window (group, new_window);
+  else
+    group_add_window (rp_current_group, new_window);
+
+  PRINT_DEBUG(("frame_num: %d\n", frame_num));
+  if (frame_num >= 0)
+    new_window->intended_frame_number = frame_num;
 
   return new_window;
 }
@@ -169,13 +257,13 @@ find_window (Window w)
 
   win = find_window_in_list (w, &rp_mapped_window);
 
-  if (!win) 
+  if (!win)
     {
       win = find_window_in_list (w, &rp_unmapped_window);
       if (win)
-	PRINT_DEBUG (("Window found in unmapped window list\n"));
+        PRINT_DEBUG (("Window found in unmapped window list\n"));
       else
-	PRINT_DEBUG (("Window not found.\n"));
+        PRINT_DEBUG (("Window not found.\n"));
     }
   else
     {
@@ -213,8 +301,8 @@ find_window_name (char *name)
 
   list_for_each_entry (cur, &rp_current_group->mapped_windows, node)
     {
-      if (str_comp (name, window_name (cur->win), strlen (name))) 
-	return cur->win;
+      if (str_comp (name, window_name (cur->win), strlen (name)))
+        return cur->win;
     }
 
   /* didn't find it */
@@ -230,14 +318,14 @@ find_window_prev (rp_window *w)
 
   if (!w) return NULL;
 
-  for (cur = list_prev_entry (w, &rp_mapped_window, node); 
-       cur != w; 
+  for (cur = list_prev_entry (w, &rp_mapped_window, node);
+       cur != w;
        cur = list_prev_entry (cur, &rp_mapped_window, node))
     {
       if (!find_windows_frame (cur))
-	{
-	  return cur;
-	}
+        {
+          return cur;
+        }
     }
 
   return NULL;
@@ -252,23 +340,23 @@ find_window_next (rp_window *w)
 
   if (!w) return NULL;
 
-  for (cur = list_next_entry (w, &rp_mapped_window, node); 
-       cur != w; 
+  for (cur = list_next_entry (w, &rp_mapped_window, node);
+       cur != w;
        cur = list_next_entry (cur, &rp_mapped_window, node))
     {
       if (!find_windows_frame (cur))
-	{
-	  return cur;
-	}
+        {
+          return cur;
+        }
     }
 
   return NULL;
 }
 
 rp_window *
-find_window_other ()
+find_window_other (rp_screen *screen)
 {
-  return group_last_window (rp_current_group);
+  return group_last_window (rp_current_group, screen);
 }
 
 
@@ -283,10 +371,10 @@ insert_into_list (rp_window *win, struct list_head *list)
   list_for_each_entry (cur, list, node)
     {
       if (cur->number > win->number)
-	{
-	  list_add_tail (&win->node, &cur->node);
-	  return;
-	}
+        {
+          list_add_tail (&win->node, &cur->node);
+          return;
+        }
     }
 
   list_add_tail(&win->node, list);
@@ -296,9 +384,9 @@ static void
 save_mouse_position (rp_window *win)
 {
   Window root_win, child_win;
-  int win_x, win_y;
+  int root_x, root_y;
   unsigned int mask;
-  
+
   /* In the case the XQueryPointer raises a BadWindow error, the
      window is not mapped or has been destroyed so it doesn't matter
      what we store in mouse_x and mouse_y since they will never be
@@ -306,8 +394,8 @@ save_mouse_position (rp_window *win)
 
   ignore_badwindow++;
 
-  XQueryPointer (dpy, win->w, &root_win, &child_win, 
-		 &win->mouse_x, &win->mouse_y, &win_x, &win_y, &mask);
+  XQueryPointer (dpy, win->w, &root_win, &child_win,
+                 &root_x, &root_y, &win->mouse_x, &win->mouse_y, &mask);
 
   ignore_badwindow--;
 }
@@ -334,10 +422,10 @@ give_window_focus (rp_window *win, rp_window *last_win)
   if (defaults.warp)
     {
       PRINT_DEBUG (("Warp pointer\n"));
-      XWarpPointer (dpy, None, win->scr->root, 
-		    0, 0, 0, 0, win->mouse_x, win->mouse_y);
+      XWarpPointer (dpy, None, win->w,
+                    0, 0, 0, 0, win->mouse_x, win->mouse_y);
     }
-  
+
   /* Swap colormaps */
   if (last_win != NULL) XUninstallColormap (dpy, last_win->colormap);
   XInstallColormap (dpy, win->colormap);
@@ -361,9 +449,9 @@ unhide_transient_for (rp_window *win)
   frame = find_windows_frame (win);
 
   transient_for = find_window (win->transient_for);
-  if (transient_for == NULL) 
+  if (transient_for == NULL)
     {
-      PRINT_DEBUG (("Can't find transient_for for '%s'", win->name ));
+      PRINT_DEBUG (("Can't find transient_for for '%s'\n", win->name ));
       return;
     }
 
@@ -373,17 +461,17 @@ unhide_transient_for (rp_window *win)
       maximize (transient_for);
 
       PRINT_DEBUG (("unhide transient window: %s\n", transient_for->name));
-      
+
       unhide_window_below (transient_for);
 
-      if (transient_for->transient) 
-	{
-	  unhide_transient_for (transient_for);
-	}
+      if (transient_for->transient)
+        {
+          unhide_transient_for (transient_for);
+        }
 
       set_frames_window (frame, win);
     }
-  else if (transient_for->transient) 
+  else if (transient_for->transient)
     {
       unhide_transient_for (transient_for);
     }
@@ -401,9 +489,9 @@ hide_transient_for_between (rp_window *win, rp_window *last)
   if (!win->transient) return;
 
   transient_for = find_window (win->transient_for);
-  if (transient_for == last) 
+  if (transient_for == last)
     {
-      PRINT_DEBUG (("Can't find transient_for for '%s'", win->name ));
+      PRINT_DEBUG (("Can't find transient_for for '%s'\n", win->name ));
       return;
     }
 
@@ -413,7 +501,7 @@ hide_transient_for_between (rp_window *win, rp_window *last)
       hide_window (transient_for);
     }
 
-  if (transient_for->transient) 
+  if (transient_for->transient)
     {
       hide_transient_for (transient_for);
     }
@@ -447,33 +535,115 @@ is_transient_ancestor (rp_window *win, rp_window *transient_for)
     {
       tmp = find_window (tmp->transient_for);
       if (tmp == transient_for)
-	return 1;
+        return 1;
     } while (tmp && tmp->transient);
-  
+
   return 0;
 }
 #endif
 
 /* In the current frame, set the active window to win. win will have focus. */
+void set_active_window (rp_window *win)
+{
+  set_active_window_body(win, 0);
+}
+
+void set_active_window_force (rp_window *win)
+{
+  set_active_window_body(win, 1);
+}
+
+/* FIXME: This function is probably a mess. I can't remember a time
+   when I didn't think this. It probably needs to be fixed up. */
 void
-set_active_window (rp_window *win)
+set_active_window_body (rp_window *win, int force)
 {
   rp_window *last_win;
-  rp_frame *frame;
+  rp_frame *frame = NULL, *last_frame = NULL;
 
   if (win == NULL) return;
 
+  PRINT_DEBUG (("intended_frame_number: %d\n", win->intended_frame_number));
+
   /* With Xinerama, we can move a window over to the current screen; otherwise
    * we have to switch to the screen that the window belongs to.
    */
   if (rp_have_xinerama)
     {
-      frame = screen_get_frame (current_screen(), current_screen()->current_frame);
+      /* use the intended frame if we can. */
+      if (win->intended_frame_number >= 0)
+        {
+          frame = screen_get_frame (current_screen(), win->intended_frame_number);
+          win->intended_frame_number = -1;
+          if (frame != current_frame())
+            last_frame = current_frame();
+        }
+
+      if (!frame)
+        frame = screen_get_frame (current_screen(), current_screen()->current_frame);
     }
   else
     {
-      frame = screen_get_frame (win->scr, win->scr->current_frame);
+      /* use the intended frame if we can. */
+      if (win->intended_frame_number >= 0)
+        {
+          frame = screen_get_frame (win->scr, win->intended_frame_number);
+          win->intended_frame_number = -1;
+          if (frame != current_frame())
+            last_frame = current_frame();
+        }
+
+      if (!frame)
+        frame = screen_get_frame (win->scr, win->scr->current_frame);
     }
+
+  if (frame->dedicated && !force)
+    {
+      /* Try to find a non-dedicated frame.  */
+      rp_frame *cur;
+      rp_screen *scr;
+      int done;
+
+      scr = (rp_have_xinerama)?&screens[rp_current_screen]:win->scr;
+      done = 0;
+
+      /* Try the only / current screen... */
+      for (cur = list_next_entry (frame, &scr->frames, node);
+           cur != frame && !done;
+           cur = list_next_entry (cur, &scr->frames, node))
+        {
+          if (!cur->dedicated)
+            {
+              set_active_frame (cur);
+              last_frame = frame;
+              frame = cur;
+              done = 1;
+            }
+        }
+
+      /* If we have Xinerama, we can check *all* screens... */
+      if (rp_have_xinerama && !done)
+        {
+          int i;
+
+          for (i=0; i<num_screens && !done; i++)
+            {
+              if (scr == &screens[i]) continue;
+              list_for_each_entry (cur,&screens[i].frames,node)
+                {
+                  if (!cur->dedicated)
+                    {
+                      set_active_frame (cur);
+                      last_frame = frame;
+                      frame = cur;
+                      done = 1; /* Break outer loop. */
+                      break;    /* Break inner loop. */
+                    }
+                }
+            }
+        }
+    }
+
   last_win = set_frames_window (frame, win);
 
   if (last_win) PRINT_DEBUG (("last window: %s\n", window_name (last_win)));
@@ -487,21 +657,18 @@ set_active_window (rp_window *win)
 
   /* The other windows in the frame will be hidden if this window
      doesn't qualify as a transient window (ie dialog box. */
-#ifdef MAXSIZE_WINDOWS_ARE_TRANSIENTS
-  if (!win->transient
-      && !(win->hints->flags & PMaxSize 
-	   && (win->hints->max_width < win->scr->width
-	       || win->hints->max_height < win->scr->height)))
-#else
-  if (!win->transient)
-#endif
+  if (!window_is_transient (win))
     hide_others(win);
 
   /* Make sure the program bar is always on the top */
-  update_window_names (win->scr);
+  update_window_names (win->scr, defaults.window_fmt);
 
   XSync (dpy, False);
 
+  /* If we switched frame, go back to the old one. */
+  if (last_frame)
+    set_active_frame (last_frame);
+
   /* Call the switch window hook */
   hook_run (&rp_switch_win_hook);
 }
@@ -529,122 +696,26 @@ goto_window (rp_window *win)
 }
 
 void
-print_window_information (rp_window *win)
+print_window_information (rp_group *group, rp_window *win)
 {
-  marked_message_printf (0, 0, MESSAGE_WINDOW_INFORMATION, 
-			 win->number, window_name (win));
-}
-
-/* format options
-   %n - Window number
-   %s - Window status (current window, last window, etc)
-   %t - Window Name
-   %a - application name
-   %c - resource class
-   %i - X11 Window ID
-   %l - last access number
-   %f - print the frame number the window is in
-   
- */
-static void
-format_window_name (char *fmt, rp_window_elem *win_elem, rp_window *other_win, 
-		    struct sbuf *buffer)
-{
-  int esc = 0;
-  char dbuf[10];
-
-  for(; *fmt; fmt++)
-    {
-      if (*fmt == '%' && !esc)
-	{
-	  esc = 1;
-	  continue;
-	}
-
-      if (esc)
-	{
-	  switch (*fmt)
-	    {
-	    case 'n':
-	      snprintf (dbuf, 10, "%d", win_elem->number);
-	      sbuf_concat (buffer, dbuf);
-	      break;
-
-	    case 's':
-	      if (win_elem->win == current_window())
-		sbuf_concat (buffer, "*");
-	      else if (win_elem->win == other_win)
-		sbuf_concat (buffer, "+");
-	      else
-		sbuf_concat (buffer, "-");
-	      break;
-
-	    case 't':
-	      sbuf_concat (buffer, window_name (win_elem->win));
-	      break;
-
-	    case 'a':
-	      if (win_elem->win->res_name)
-		sbuf_concat (buffer, win_elem->win->res_name);
-	      else
-		sbuf_concat (buffer, "None");
-	      break;
-
-	    case 'c':
-	      if (win_elem->win->res_class)
-		sbuf_concat (buffer, win_elem->win->res_class);
-	      else
-		sbuf_concat (buffer, "None");
-	      break;
-
-	    case 'i':
-	      snprintf (dbuf, 9, "%ld", (unsigned long)win_elem->win->w);
-	      sbuf_concat (buffer, dbuf);
-	      break;
-
-	    case 'l':
-	      snprintf (dbuf, 9, "%d", win_elem->win->last_access);
-	      sbuf_concat (buffer, dbuf);
-	      break;
-
-	    case 'f':
-	      if (win_elem->win->frame_number != EMPTY)
-		{
-		  snprintf (dbuf, 9, "%d", win_elem->win->frame_number);
-		  sbuf_concat (buffer, dbuf);
-		}
-	      else
-		sbuf_concat (buffer, " ");
-	      break;
-
-	    case '%':
-	      sbuf_concat (buffer, "%");
-	      break;
-
-	    default:
-	      sbuf_printf_concat (buffer, "%%%c", *fmt);
-	      break;
-	    }
-
-	  /* Reset the 'escape' state. */
-	  esc = 0;
-	}
-      else
-	{
-	  /* Insert the character. */
-	  dbuf[0] = *fmt;
-	  dbuf[1] = 0;
-	  sbuf_concat (buffer, dbuf);
-	}
-    }
+  rp_window_elem *win_elem;
+  /* Display the window's number in group. This gives the possibility
+     of windows existing in multiple groups. */
+  win_elem = group_find_window (&group->mapped_windows, win);
+  if (win_elem)
+    marked_message_printf (0, 0, MESSAGE_WINDOW_INFORMATION,
+                           win_elem->number, window_name (win));
+  else
+    marked_message_printf (0, 0, "%s doesn't exist in group %d\n",
+                           window_name(win), group->number);
 }
 
 /* get the window list and store it in buffer delimiting each window
    with delim. mark_start and mark_end will be filled with the text
    positions for the start and end of the current window. */
 void
-get_window_list (char *fmt, char *delim, struct sbuf *buffer, 
-		 int *mark_start, int *mark_end)
+get_window_list (char *fmt, char *delim, struct sbuf *buffer,
+                 int *mark_start, int *mark_end)
 {
   rp_window_elem *we;
   rp_window *other_window;
@@ -652,7 +723,7 @@ get_window_list (char *fmt, char *delim, struct sbuf *buffer,
   if (buffer == NULL) return;
 
   sbuf_clear (buffer);
-  other_window = find_window_other ();
+  other_window = find_window_other (current_screen());
 
   /* We only loop through the current group to look for windows. */
   list_for_each_entry (we,&rp_current_group->mapped_windows,node)
@@ -660,29 +731,29 @@ get_window_list (char *fmt, char *delim, struct sbuf *buffer,
       PRINT_DEBUG (("%d-%s\n", we->number, window_name (we->win)));
 
       if (we->win == current_window())
-	*mark_start = strlen (sbuf_get (buffer));
+        *mark_start = strlen (sbuf_get (buffer));
 
       /* A hack, pad the window with a space at the beginning and end
          if there is no delimiter. */
       if (!delim)
-	sbuf_concat (buffer, " ");
+        sbuf_concat (buffer, " ");
 
-      format_window_name (fmt, we, other_window, buffer);
+      format_string (fmt, we, buffer);
 
       /* A hack, pad the window with a space at the beginning and end
          if there is no delimiter. */
       if (!delim)
-	sbuf_concat (buffer, " ");
+        sbuf_concat (buffer, " ");
 
       /* Only put the delimiter between the windows, and not after the the last
          window. */
       if (delim && we->node.next != &rp_current_group->mapped_windows)
-	sbuf_concat (buffer, delim);
+        sbuf_concat (buffer, delim);
 
-      if (we->win == current_window()) 
-	{
-	  *mark_end = strlen (sbuf_get (buffer));
-	}
+      if (we->win == current_window())
+        {
+          *mark_end = strlen (sbuf_get (buffer));
+        }
     }
 
   if (!strcmp (sbuf_get (buffer), ""))
